@@ -3,6 +3,7 @@
 #'
 #' @param DataSet \code{data.frame} A data set in Matrix Form.
 #' @param colYears \code{string or integer} A name or the position of the 'Years' column just in case that is not the first column.
+#' @param colID \code{string or integer} A name or the position of the 'ID' column, just in case that is not the second column.
 #' @param Years.testing \code{vector} A vector with the names of the years to use in test.
 #' @param Traits.testing \code{vector} A vector with the names of the traits to use in test.
 #' @param dec \code{integer} Number of decimals to print in the results.
@@ -23,13 +24,15 @@
 #' }
 #'
 #' @export
-IBCF.Years <- function(DataSet, colYears = 1, Years.testing = '', Traits.testing = '', dec = 4) {
+IBCF.Years <- function(DataSet, colYears = 1, colID = 2, Years.testing = '', Traits.testing = '', dec = 4) {
   DataSet[, colYears] <- as.character(DataSet[, colYears]) #No factors admited
   No.Years <- length(unique(DataSet[, colYears]))
   No.Years.testing <- length(Years.testing)
   No.Traits <- length(colnames(DataSet)) - 2
   No.Traits.testing <- length(Traits.testing)
   No.Traits.Accepted <- round(0.7*No.Traits)
+  colYears <- ifelse(is.numeric(colYears), colYears, which(names(DataSet) == colYears))
+  colID <- ifelse(is.numeric(colID), colID, which(names(DataSet) == colID))
 
   if (No.Years.testing == No.Years) {
     stop("No.Years.testing must be less than No.Years in the whole data set") }
@@ -44,35 +47,33 @@ IBCF.Years <- function(DataSet, colYears = 1, Years.testing = '', Traits.testing
   Data.trn <- DataSet
   Data.trn[pos.Years.testing, pos.Traits.testing] <- NA
 
-  rows.Na <- which(apply(Data.trn,1,function(x)any(is.na(x))) == TRUE)
+  rows.Na <- which(apply(Data.trn[, -c(colYears, colID)],1,function(x)any(is.na(x))) == TRUE)
 
-  Means_trn <- apply(Data.trn[,-c(1,2)], 2, mean, na.rm = T)
-  SDs_trn <- apply(Data.trn[,-c(1,2)], 2, sd, na.rm = T)
+  Means_trn <- apply(Data.trn[, -c(colYears, colID)], 2, mean, na.rm = T)
+  SDs_trn <- apply(Data.trn[, -c(colYears, colID)], 2, sd, na.rm = T)
 
-  # Mean_and_SD <- data.frame(cbind(Means_trn, SDs_trn))
-
-  Scaled_Col <- scale(Data.trn[,-c(1,2)])
+  Scaled_Col <- scale(Data.trn[, -c(colYears, colID)])
   Means_trn_Row <- apply(Scaled_Col, 1, mean, na.rm = T)
   SDs_trn_Row <- apply(Scaled_Col, 1, sd, na.rm = T)
 
   if (any(is.na(SDs_trn_Row))) {
-    Data.trn_scaled <- data.frame(ID = as.character(Data.trn[, c(1)]), Scaled_Col)
+    Data.trn_scaled <- data.frame(ID = as.character(Data.trn[, colID]), Scaled_Col)
   } else {
     Scaled_Row <- t(scale(t(Scaled_Col)))
-    Data.trn_scaled <- data.frame(ID = as.character(Data.trn[, c(1)]), Scaled_Row)
+    Data.trn_scaled <- data.frame(ID = as.character(Data.trn[, colID]), Scaled_Row)
   }
 
-  ratings <- Data.trn_scaled[,-1]
+  ratings <- within(Data.trn_scaled, rm('ID'))
 
   x <- ratings
   x[is.na(x)] <- 0
   item_sim <- lsa::cosine(as.matrix((x)))
 
-  Hyb.pred <- as.data.frame(Data.trn_scaled[,-1])
+  Hyb.pred <- within(Data.trn_scaled, rm('ID'))
 
-  for (i in 1:length(rows.Na)) {
+  for (i in seq_len(length(rows.Na))) {
     pos <- rows.Na[i]
-    Hyb.pred[pos,] <- c(rec_itm_for_geno(pos,item_sim,ratings))
+    Hyb.pred[pos,] <- c(rec_itm_for_geno(pos, item_sim, ratings))
   }
 
   All.Pred <- data.matrix(Hyb.pred)
@@ -86,25 +87,25 @@ IBCF.Years <- function(DataSet, colYears = 1, Years.testing = '', Traits.testing
     All.Pred_O <- sapply(1:ncol(All.Pred_O_Row), function(i) (All.Pred_O_Row[,i]*SDs_trn[i] + Means_trn[i]))
   }
 
-  All.Pred_O <- data.frame(DataSet[, c(1,2)], All.Pred_O)
-    colnames(All.Pred_O) <- c(colnames(DataSet[, c(1,2)]), colnames(Data.trn_scaled[, -1]))
+  All.Pred_O <- data.frame(DataSet[, c(colYears, colID)], All.Pred_O)
+    colnames(All.Pred_O) <- c(colnames(DataSet[, c(colYears, colID)]), colnames(Data.trn_scaled[, -1]))
 
-  Data.Obs <- getTidyForm(DataSet[, c(1, 2, pos.Traits.testing)], onlyTrait = T)$Response
-  Data.Pred <- getTidyForm(All.Pred_O[, c(1, 2, pos.Traits.testing)], onlyTrait = T)$Response
-  Data.Obs_Pred <- data.frame(DataSet[pos.Years.testing, c(1, 2, pos.Traits.testing)],
+  Data.Obs <- getTidyForm(DataSet[, c(colYears, colID, pos.Traits.testing)], onlyTrait = T)$Response
+  Data.Pred <- getTidyForm(All.Pred_O[, c(colYears, colID, pos.Traits.testing)], onlyTrait = T)$Response
+  Data.Obs_Pred <- data.frame(DataSet[pos.Years.testing, c(colYears, colID, pos.Traits.testing)],
                               All.Pred_O[pos.Years.testing, pos.Traits.testing])
-  colnames(Data.Obs_Pred) <- c(colnames(DataSet)[ c(1, 2, pos.Traits.testing)], paste0(colnames(All.Pred_O)[pos.Traits.testing],'.1'))
+  colnames(Data.Obs_Pred) <- c(colnames(DataSet)[ c(colYears, colID, pos.Traits.testing)], paste0(colnames(All.Pred_O)[pos.Traits.testing],'.1'))
   Pearson <- c()
   MSEP <- c()
   Year_Trait <- c()
 
-  for (q in 1:length(Years.testing)) {
+  for (q in seq_len(length(Years.testing))) {
     pos.Years_q <- which(c(DataSet[, colYears]) == Years.testing[q])
 
     DataSet_tst <- DataSet[pos.Years_q, pos.Traits.testing]
     All.Pred_O_tst <- All.Pred_O[pos.Years_q, pos.Traits.testing]
 
-    Cor_all_tst <- cor(DataSet_tst,All.Pred_O_tst, use = "pairwise.complete.obs")
+    Cor_all_tst <- cor(DataSet_tst, All.Pred_O_tst, use = "pairwise.complete.obs")
 
     Dif_Obs_pred <- DataSet_tst - All.Pred_O_tst
     Dif_Obs_pred2 <- Dif_Obs_pred^2
